@@ -12,7 +12,7 @@ distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
-]]--
+]] --
 
 --[[[
 -- @module lua_magic/heuristics
@@ -28,18 +28,18 @@ local fun = require "fun"
 local N = "lua_magic"
 local msoffice_trie
 local msoffice_patterns = {
-  doc = {[[WordDocument]]},
-  xls = {[[Workbook]], [[Book]]},
-  ppt = {[[PowerPoint Document]], [[Current User]]},
-  vsd = {[[VisioDocument]]},
+  doc = { [[WordDocument]] },
+  xls = { [[Workbook]], [[Book]] },
+  ppt = { [[PowerPoint Document]], [[Current User]] },
+  vsd = { [[VisioDocument]] },
 }
 local msoffice_trie_clsid
 local msoffice_clsids = {
-  doc = {[[0609020000000000c000000000000046]]},
-  xls = {[[1008020000000000c000000000000046]], [[2008020000000000c000000000000046]]},
-  ppt = {[[108d81649b4fcf1186ea00aa00b929e8]]},
-  msg = {[[46f0060000000000c000000000000046]], [[0b0d020000000000c000000000000046]]},
-  msi = {[[84100c0000000000c000000000000046]]},
+  doc = { [[0609020000000000c000000000000046]] },
+  xls = { [[1008020000000000c000000000000046]], [[2008020000000000c000000000000046]] },
+  ppt = { [[108d81649b4fcf1186ea00aa00b929e8]] },
+  msg = { [[46f0060000000000c000000000000046]], [[0b0d020000000000c000000000000046]] },
+  msi = { [[84100c0000000000c000000000000046]] },
 }
 local zip_trie
 local zip_patterns = {
@@ -54,37 +54,37 @@ local zip_patterns = {
     [[mimetypeapplication/vnd\.oasis\.opendocument\.formula]],
     [[mimetypeapplication/vnd\.oasis\.opendocument\.chart]]
   },
-  odp = {[[mimetypeapplication/vnd\.oasis\.opendocument\.presentation]]},
-  epub = {[[epub\+zip]]},
-  asice = {[[mimetypeapplication/vnd\.etsi\.asic-e\+zipPK]]},
-  asics = {[[mimetypeapplication/vnd\.etsi\.asic-s\+zipPK]]},
+  odp = { [[mimetypeapplication/vnd\.oasis\.opendocument\.presentation]] },
+  epub = { [[epub\+zip]] },
+  asice = { [[mimetypeapplication/vnd\.etsi\.asic-e\+zipPK]] },
+  asics = { [[mimetypeapplication/vnd\.etsi\.asic-s\+zipPK]] },
 }
 
 local txt_trie
 local txt_patterns = {
   html = {
-    {[=[(?i)<html[\s>]]=], 32},
-    {[[(?i)<script\b]], 20}, -- Commonly used by spammers
-    {[[<script\s+type="text\/javascript">]], 31}, -- Another spammy pattern
-    {[[(?i)<\!DOCTYPE HTML\b]], 33},
-    {[[(?i)<body\b]], 20},
-    {[[(?i)<table\b]], 20},
-    {[[(?i)<a\s]], 10},
-    {[[(?i)<p\b]], 10},
-    {[[(?i)<div\b]], 10},
-    {[[(?i)<span\b]], 10},
+    { [=[(?i)<html[\s>]]=],                   32 },
+    { [[(?i)<script\b]],                      20 }, -- Commonly used by spammers
+    { [[<script\s+type="text\/javascript">]], 31 }, -- Another spammy pattern
+    { [[(?i)<\!DOCTYPE HTML\b]],              33 },
+    { [[(?i)<body\b]],                        20 },
+    { [[(?i)<table\b]],                       20 },
+    { [[(?i)<a\s]],                           10 },
+    { [[(?i)<p\b]],                           10 },
+    { [[(?i)<div\b]],                         10 },
+    { [[(?i)<span\b]],                        10 },
   },
   csv = {
-    {[[(?:[-a-zA-Z0-9_]+\s*,){2,}(?:[-a-zA-Z0-9_]+,?[ ]*[\r\n])]], 20}
+    { [[(?:[-a-zA-Z0-9_]+\s*,){2,}(?:[-a-zA-Z0-9_]+,?[ ]*[\r\n])]], 20 }
   },
   ics = {
-    {[[^BEGIN:VCALENDAR\r?\n]], 40},
+    { [[^BEGIN:VCALENDAR\r?\n]], 40 },
   },
   vcf = {
-    {[[^BEGIN:VCARD\r?\n]], 40},
+    { [[^BEGIN:VCARD\r?\n]], 40 },
   },
   xml = {
-    {[[<\?xml\b.+\?>]], 31},
+    { [[<\?xml\b.+\?>]], 31 },
   }
 }
 
@@ -98,16 +98,16 @@ local exports = {}
 
 local function compile_tries()
   local default_compile_flags = bit.bor(rspamd_trie.flags.re,
-      rspamd_trie.flags.dot_all,
-      rspamd_trie.flags.single_match,
-      rspamd_trie.flags.no_start)
+    rspamd_trie.flags.dot_all,
+    rspamd_trie.flags.single_match,
+    rspamd_trie.flags.no_start)
   local function compile_pats(patterns, indexes, transform_func, compile_flags)
     local strs = {}
-    for ext,pats in pairs(patterns) do
-      for _,pat in ipairs(pats) do
+    for ext, pats in pairs(patterns) do
+      for _, pat in ipairs(pats) do
         -- These are utf16 strings in fact...
         strs[#strs + 1] = transform_func(pat)
-        indexes[#indexes + 1] = {ext, pat}
+        indexes[#indexes + 1] = { ext, pat }
       end
     end
 
@@ -119,13 +119,15 @@ local function compile_tries()
     local function msoffice_pattern_transform(pat)
       return '^' ..
           table.concat(
-              fun.totable(
-                  fun.map(function(c) return c .. [[\x{00}]] end,
-                      fun.iter(pat))))
+            fun.totable(
+              fun.map(function(c)
+                  return c .. [[\x{00}]]
+                end,
+                fun.iter(pat))))
     end
     local function msoffice_clsid_transform(pat)
       local hex_table = {}
-      for i=1,#pat,2 do
+      for i = 1, #pat, 2 do
         local subc = pat:sub(i, i + 1)
         hex_table[#hex_table + 1] = string.format('\\x{%s}', subc)
       end
@@ -134,19 +136,23 @@ local function compile_tries()
     end
     -- Directory entries
     msoffice_trie = compile_pats(msoffice_patterns, msoffice_patterns_indexes,
-        msoffice_pattern_transform)
+      msoffice_pattern_transform)
     -- Clsids
     msoffice_trie_clsid = compile_pats(msoffice_clsids, msoffice_clsid_indexes,
-        msoffice_clsid_transform)
+      msoffice_clsid_transform)
     -- Misc zip patterns at the initial fragment
     zip_trie = compile_pats(zip_patterns, zip_patterns_indexes,
-        function(pat) return pat end)
+      function(pat)
+        return pat
+      end)
     -- Text patterns at the initial fragment
     txt_trie = compile_pats(txt_patterns, txt_patterns_indexes,
-        function(pat_tbl) return pat_tbl[1] end,
-        bit.bor(rspamd_trie.flags.re,
-            rspamd_trie.flags.dot_all,
-            rspamd_trie.flags.no_start))
+      function(pat_tbl)
+        return pat_tbl[1]
+      end,
+      bit.bor(rspamd_trie.flags.re,
+        rspamd_trie.flags.dot_all,
+        rspamd_trie.flags.no_start))
   end
 end
 
@@ -160,12 +166,13 @@ local function detect_ole_format(input, log_obj, _, part)
     return nil
   end
 
-  local bom,sec_size = rspamd_util.unpack('<I2<I2', input:span(29, 4))
+  local bom, sec_size = rspamd_util.unpack('<I2<I2', input:span(29, 4))
   if bom == 0xFFFE then
     bom = '<'
   else
     lua_util.debugm(N, log_obj, "bom file!: %s", bom)
-    bom = '>'; sec_size = bit.bswap(sec_size)
+    bom = '>';
+    sec_size = bit.bswap(sec_size)
   end
 
   if sec_size < 7 or sec_size > 31 then
@@ -194,39 +201,39 @@ local function detect_ole_format(input, log_obj, _, part)
         -- Extract clsid
         local matches = msoffice_trie_clsid:match(input:span(offset + 80, 16))
         if matches then
-          for n,_ in pairs(matches) do
+          for n, _ in pairs(matches) do
             if msoffice_clsid_indexes[n] then
               lua_util.debugm(N, log_obj, "found valid clsid for %s",
-                  msoffice_clsid_indexes[n][1])
-              return true,msoffice_clsid_indexes[n][1]
+                msoffice_clsid_indexes[n][1])
+              return true, msoffice_clsid_indexes[n][1]
             end
           end
         end
-        return true,nil
+        return true, nil
       elseif dtype == 2 then
         local matches = msoffice_trie:match(input:span(offset, 64))
         if matches then
-          for n,_ in pairs(matches) do
+          for n, _ in pairs(matches) do
             if msoffice_patterns_indexes[n] then
-              return true,msoffice_patterns_indexes[n][1]
+              return true, msoffice_patterns_indexes[n][1]
             end
           end
         end
-        return true,nil
+        return true, nil
       elseif dtype >= 0 and dtype < 5 then
         -- Bad type
-        return true,nil
+        return true, nil
       end
     end
 
-    return false,nil
+    return false, nil
   end
 
   repeat
-    local res,ext = process_dir_entry(directory_offset)
+    local res, ext = process_dir_entry(directory_offset)
 
     if res and ext then
-      return ext,60
+      return ext, 60
     end
 
     if not res then
@@ -247,7 +254,7 @@ local function process_top_detected(res)
       return res[ex1] > res[ex2]
     end)
 
-    return extensions[1],res[extensions[1]]
+    return extensions[1], res[extensions[1]]
   end
 
   return nil
@@ -266,6 +273,15 @@ local function detect_archive_flaw(part, arch, log_obj, _)
     apk = 0,
   } -- ext + confidence pairs
 
+  local function has_control_or_zw(fname)
+    -- control ASCII
+    if fname:find("[%z\1-\31]") then return true end
+    -- common zero-width UTF-8: U+200B..U+200D, U+FEFF
+    if fname:find("\226\128[\139-\141]") then return true end -- U+200B..U+200D
+    if fname:find("\239\187\191") then return true end        -- U+FEFF
+    return false
+  end
+
   -- General msoffice patterns
   local function add_msoffice_confidence(incr)
     res.docx = res.docx + incr
@@ -276,7 +292,10 @@ local function detect_archive_flaw(part, arch, log_obj, _)
   if arch_type == 'zip' then
     -- Find specific files/folders in zip file
     local files = arch:get_files(100) or {}
-    for _,file in ipairs(files) do
+    for _, file in ipairs(files) do
+      if has_control_or_zw(file) then
+        lua_util.debugm(N, log_obj, "archive filename has control/zw chars: %s", file)
+      end
       if file == '[Content_Types].xml' then
         add_msoffice_confidence(10)
       elseif file:sub(1, 3) == 'xl/' then
@@ -292,10 +311,10 @@ local function detect_archive_flaw(part, arch, log_obj, _)
       end
     end
 
-    local ext,weight = process_top_detected(res)
+    local ext, weight = process_top_detected(res)
 
     if weight >= 40 then
-      return ext,weight
+      return ext, weight
     end
 
     -- Apply misc Zip detection logic
@@ -306,32 +325,34 @@ local function detect_archive_flaw(part, arch, log_obj, _)
 
       local matches = zip_trie:match(start_span)
       if matches then
-        for n,_ in pairs(matches) do
+        for n, _ in pairs(matches) do
           if zip_patterns_indexes[n] then
             lua_util.debugm(N, log_obj, "found zip pattern for %s",
-                zip_patterns_indexes[n][1])
-            return zip_patterns_indexes[n][1],40
+              zip_patterns_indexes[n][1])
+            return zip_patterns_indexes[n][1], 40
           end
         end
       end
     end
   end
 
-  return arch_type:lower(),40
+  return arch_type:lower(), 40
 end
 
 local csv_grammar
 -- Returns a grammar that will count commas
 local function get_csv_grammar()
   if not csv_grammar then
-    local lpeg = require'lpeg'
+    local lpeg = require 'lpeg'
 
-    local field = '"' * lpeg.Cs(((lpeg.P(1) - '"') + lpeg.P'""' / '"')^0) * '"' +
-        lpeg.C((1 - lpeg.S',\n"')^0)
+    local field = '"' * lpeg.Cs(((lpeg.P(1) - '"') + lpeg.P '""' / '"') ^ 0) * '"' +
+        lpeg.C((1 - lpeg.S ',\n"') ^ 0)
 
-    csv_grammar = lpeg.Cf(lpeg.Cc(0) * field * lpeg.P( (lpeg.P(',') +
-        lpeg.P('\t')) * field)^1 * (lpeg.S'\r\n' + -1),
-        function(acc) return acc + 1 end)
+    csv_grammar = lpeg.Cf(lpeg.Cc(0) * field * lpeg.P((lpeg.P(',') +
+        lpeg.P('\t')) * field) ^ 1 * (lpeg.S '\r\n' + -1),
+      function(acc)
+        return acc + 1
+      end)
   end
 
   return csv_grammar
@@ -351,14 +372,14 @@ local function validate_csv(part, content, log_obj)
 
     if not ncommas then
       lua_util.debugm(N, log_obj, "not a csv line at line number %s",
-          matched_lines)
+        matched_lines)
       return false
     end
 
     if expected_commas and ncommas ~= expected_commas then
       -- Mismatched commas
       lua_util.debugm(N, log_obj, "missmatched commas on line %s: %s != %s",
-          matched_lines, ncommas, expected_commas)
+        matched_lines, ncommas, expected_commas)
       return false
     elseif not expected_commas then
       if ncommas == 0 then
@@ -376,7 +397,7 @@ local function validate_csv(part, content, log_obj)
   end
 
   lua_util.debugm(N, log_obj, "csv content is sane: %s fields; %s lines checked",
-      expected_commas, matched_lines)
+    expected_commas, matched_lines)
 
   return true
 end
@@ -402,17 +423,17 @@ exports.text_part_heuristic = function(part, log_obj, _)
       while b >= 127 and idx < len do
         -- utf8 part
         if bit.band(b, 0xe0) == 0xc0 and remain > 1 and
-                bit.band(bytes[idx + 1], 0xc0) == 0x80 then
-          return true,1
+            bit.band(bytes[idx + 1], 0xc0) == 0x80 then
+          return true, 1
         elseif bit.band(b, 0xf0) == 0xe0 and remain > 2 and
-                bit.band(bytes[idx + 1], 0xc0) == 0x80 and
-                bit.band(bytes[idx + 2], 0xc0) == 0x80 then
-          return true,2
+            bit.band(bytes[idx + 1], 0xc0) == 0x80 and
+            bit.band(bytes[idx + 2], 0xc0) == 0x80 then
+          return true, 2
         elseif bit.band(b, 0xf8) == 0xf0 and remain > 3 and
-                bit.band(bytes[idx + 1], 0xc0) == 0x80 and
-                bit.band(bytes[idx + 2], 0xc0) == 0x80 and
-                bit.band(bytes[idx + 3], 0xc0) == 0x80 then
-          return true,3
+            bit.band(bytes[idx + 1], 0xc0) == 0x80 and
+            bit.band(bytes[idx + 2], 0xc0) == 0x80 and
+            bit.band(bytes[idx + 3], 0xc0) == 0x80 then
+          return true, 3
         end
 
         n8bit = n8bit + 1
@@ -422,10 +443,10 @@ exports.text_part_heuristic = function(part, log_obj, _)
       end
 
       if n8bit >= 3 then
-        return true,n8bit
+        return true, n8bit
       end
 
-      return false,0
+      return false, 0
     end
 
     -- Convert to string as LuaJIT can optimise string.sub (and fun.iter) but not C calls
@@ -439,7 +460,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
       if (b < 0x20) and not (b == 0x0d or b == 0x0a or b == 0x09) then
         non_printable = non_printable + 1
       elseif b >= 127 then
-        local c,nskip = rough_8bit_check(bytes, i, tlen - i, tlen)
+        local c, nskip = rough_8bit_check(bytes, i, tlen - i, tlen)
 
         if not c then
           non_printable = non_printable + 1
@@ -451,7 +472,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
     until i > tlen
 
     lua_util.debugm(N, log_obj, "text part check: %s printable, %s non-printable, %s total",
-        tlen - non_printable, non_printable, tlen)
+      tlen - non_printable, non_printable, tlen)
     if non_printable / tlen > 0.0078125 then
       return false
     end
@@ -462,7 +483,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
   local parent = part:get_parent()
 
   if parent then
-    local parent_type,parent_subtype = parent:get_type()
+    local parent_type, parent_subtype = parent:get_type()
 
     if parent_type == 'multipart' and parent_subtype == 'encrypted' then
       -- Skip text heuristics for encrypted parts
@@ -473,7 +494,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
   end
 
   local content = part:get_content()
-  local mtype,msubtype = part:get_type()
+  local mtype, msubtype = part:get_type()
   local clen = #content
   local is_text
 
@@ -495,16 +516,16 @@ exports.text_part_heuristic = function(part, log_obj, _)
 
       if matches then
         -- Require at least 2 occurrences of those patterns
-        for n,positions in pairs(matches) do
-          local ext,weight = txt_patterns_indexes[n][1], txt_patterns_indexes[n][2][2]
+        for n, positions in pairs(matches) do
+          local ext, weight = txt_patterns_indexes[n][1], txt_patterns_indexes[n][2][2]
           if ext then
             res[ext] = (res[ext] or 0) + weight * #positions
             lua_util.debugm(N, log_obj, "found txt pattern for %s: %s, total: %s; %s/%s announced",
-                ext, weight * #positions, res[ext], mtype, msubtype)
+              ext, weight * #positions, res[ext], mtype, msubtype)
           end
         end
 
-        if res.html and res.html >= 40  then
+        if res.html and res.html >= 40 then
           -- HTML has priority over something like js...
           return 'html', res.html
         end
@@ -525,7 +546,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
 
       -- Content type stuff
       if (mtype == 'text' or mtype == 'application') and
-              (msubtype == 'html' or msubtype == 'xhtml+xml') then
+          (msubtype == 'html' or msubtype == 'xhtml+xml') then
         return 'html', 21
       end
 
@@ -539,12 +560,12 @@ exports.text_part_heuristic = function(part, log_obj, _)
       local function has_extension(file, ext)
         local ext_len = ext:len()
         return file:len() > ext_len + 1
-                and file:sub(-ext_len):lower() == ext
-                and file:sub(-ext_len - 1, -ext_len - 1) == '.'
+            and file:sub(-ext_len):lower() == ext
+            and file:sub(-ext_len - 1, -ext_len - 1) == '.'
       end
 
       if fname and (has_extension(fname, 'htm') or has_extension(fname, 'html')) then
-        return 'html',21
+        return 'html', 21
       end
 
       if mtype ~= 'text' then
@@ -552,7 +573,7 @@ exports.text_part_heuristic = function(part, log_obj, _)
         return nil
       end
 
-      return 'txt',40
+      return 'txt', 40
     end
   end
 end
@@ -569,7 +590,7 @@ exports.pdf_format_heuristic = function(input, log_obj, pos, part)
     weight = weight + 30
   end
 
-  return 'pdf',weight
+  return 'pdf', weight
 end
 
 exports.pe_part_heuristic = function(input, log_obj, pos, part)
@@ -590,7 +611,7 @@ exports.pe_part_heuristic = function(input, log_obj, pos, part)
     return
   end
 
-  return 'exe',30
+  return 'exe', 30
 end
 
 return exports
