@@ -1,5 +1,5 @@
 --[[
-Copyright (c) 2018, Vsevolod Stakhov <vsevolod@highsecure.ru>
+Copyright (c) 2022, Vsevolod Stakhov <vsevolod@rspamd.com>
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -12,12 +12,13 @@ distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
-]]--
+]] --
 
 local argparse = require "argparse"
 local ansicolors = require "ansicolors"
 local rspamd_util = require "rspamd_util"
 local rspamd_task = require "rspamd_task"
+local rspamd_text = require "rspamd_text"
 local rspamd_logger = require "rspamd_logger"
 local lua_meta = require "lua_meta"
 local rspamd_url = require "rspamd_url"
@@ -41,7 +42,9 @@ parser:mutex(
     parser:flag "-j --json"
           :description "JSON output",
     parser:flag "-U --ucl"
-          :description "UCL output"
+          :description "UCL output",
+    parser:flag "-M --messagepack"
+          :description "MessagePack output"
 )
 parser:flag "-C --compact"
       :description "Use compact format"
@@ -58,18 +61,20 @@ extract:argument "file"
 
 extract:flag "-t --text"
        :description "Extracts plain text data from a message"
+extract:flag "-r --raw"
+       :description "Load as raw file"
 extract:flag "-H --html"
        :description "Extracts htm data from a message"
 extract:option "-o --output"
        :description "Output format ('raw', 'content', 'oneline', 'decoded', 'decoded_utf')"
        :argname("<type>")
        :convert {
-          raw = "raw",
-          content = "content",
-          oneline = "content_oneline",
-          decoded = "raw_parsed",
-          decoded_utf = "raw_utf"
-       }
+  raw = "raw",
+  content = "content",
+  oneline = "content_oneline",
+  decoded = "raw_parsed",
+  decoded_utf = "raw_utf"
+}
        :default "content"
 extract:flag "-w --words"
        :description "Extracts words"
@@ -77,20 +82,21 @@ extract:flag "-p --part"
        :description "Show part info"
 extract:flag "-s --structure"
        :description "Show structure info (e.g. HTML tags)"
+extract:flag "-i --invisible"
+       :description "Show invisible content for HTML parts"
 extract:option "-F --words-format"
        :description "Words format ('stem', 'norm', 'raw', 'full')"
        :argname("<type>")
        :convert {
-          stem = "stem",
-          norm = "norm",
-          raw = "raw",
-          full = "full",
-       }
+  stem = "stem",
+  norm = "norm",
+  raw = "raw",
+  full = "full",
+}
        :default "stem"
 
-
 local stat = parser:command "stat st s"
-                    :description "Extracts statistical data from MIME messages"
+                   :description "Extracts statistical data from MIME messages"
 stat:argument "file"
     :description "File to process"
     :argname "<file>"
@@ -118,7 +124,7 @@ urls:mutex(
     urls:flag "-H --host"
         :description "Get hosts only",
     urls:flag "-f --full"
-      :description "Show piecewise urls as processed by Rspamd"
+        :description "Show piecewise urls as processed by Rspamd"
 )
 
 urls:flag "-u --unique"
@@ -131,7 +137,7 @@ urls:flag "-r --reverse"
     :description "Reverse sort order"
 
 local modify = parser:command "modify mod m"
-                   :description "Modifies MIME message"
+                     :description "Modifies MIME message"
 modify:argument "file"
       :description "File to process"
       :argname "<file>"
@@ -156,12 +162,54 @@ modify:option "-H --html-footer"
       :description "Adds footer to text/html parts from a specific file"
       :argname "<file>"
 
+local strip = parser:command "strip"
+                    :description "Strip attachments from a message"
+strip:argument "file"
+     :description "File to process"
+     :argname "<file>"
+     :args "+"
+strip:flag "-i --keep-images"
+     :description "Keep images"
+strip:option "--min-text-size"
+     :description "Minimal text size to keep"
+     :argname "<size>"
+     :convert(tonumber)
+     :default(0)
+strip:option "--max-text-size"
+     :description "Max text size to keep"
+     :argname "<size>"
+     :convert(tonumber)
+     :default(math.huge)
+
+local anonymize = parser:command "anonymize"
+                        :description "Try to remove sensitive information from a message"
+anonymize:argument "file"
+         :description "File to process"
+         :argname "<file>"
+         :args "+"
+anonymize:option "--exclude-header -X"
+         :description "Exclude specific headers from anonymization"
+         :argname "<header>"
+         :count "*"
+anonymize:option "--include-header -I"
+         :description "Include specific headers from anonymization"
+         :argname "<header>"
+         :count "*"
+anonymize:flag "--gpt"
+         :description "Use LLM model for anonymization (requires GPT plugin to be configured)"
+anonymize:option "--model"
+         :description "Model to use for anonymization"
+         :argname "<model>"
+anonymize:option "--prompt"
+         :description "Prompt to use for anonymization"
+         :argname "<prompt>"
+
 local sign = parser:command "sign"
-                     :description "Performs DKIM signing"
+                   :description "Performs DKIM signing"
 sign:argument "file"
-      :description "File to process"
-      :argname "<file>"
-      :args "+"
+    :description "File to process"
+    :argname "<file>"
+    :args "+"
 
 sign:option "-d --domain"
     :description "Use specific domain"
@@ -175,36 +223,75 @@ sign:option "-k --key"
     :description "Use specific key of file"
     :argname "<key>"
     :count "1"
-sign:option "-t type"
+sign:option "-t --type"
     :description "ARC or DKIM signing"
     :argname("<arc|dkim>")
     :convert {
-      ['arc'] = 'arc',
-      ['dkim'] = 'dkim',
-    }
+  ['arc'] = 'arc',
+  ['dkim'] = 'dkim',
+}
     :default 'dkim'
 sign:option "-o --output"
     :description "Output format"
     :argname("<message|signature>")
     :convert {
-      ['message'] = 'message',
-      ['signature'] = 'signature',
-    }
+  ['message'] = 'message',
+  ['signature'] = 'signature',
+}
     :default 'message'
 
-local function load_config(opts)
-  local _r,err = rspamd_config:load_ucl(opts['config'])
+local dump = parser:command "dump"
+                   :description "Dumps a raw message in different formats"
+dump:argument "file"
+    :description "File to process"
+    :argname "<file>"
+    :args "+"
+-- Duplicate format for convenience
+dump:mutex(
+    parser:flag "-j --json"
+          :description "JSON output",
+    parser:flag "-U --ucl"
+          :description "UCL output",
+    parser:flag "-M --messagepack"
+          :description "MessagePack output"
+)
+dump:flag "-s --split"
+    :description "Split the output file contents such that no content is embedded"
+
+dump:option "-o --outdir"
+    :description "Output directory"
+    :argname("<directory>")
+
+local function load_config(opts, load_tokenizers)
+  local _r, err = rspamd_config:load_ucl(opts['config'])
 
   if not _r then
     rspamd_logger.errx('cannot parse %s: %s', opts['config'], err)
     os.exit(1)
   end
 
-  _r,err = rspamd_config:parse_rcl({'logging', 'worker'})
+  _r, err = rspamd_config:parse_rcl({ 'logging', 'worker' })
   if not _r then
     rspamd_logger.errx('cannot process %s: %s', opts['config'], err)
     os.exit(1)
   end
+
+  -- Load custom tokenizers if requested
+  if load_tokenizers then
+    local success, tokenizer_err = rspamd_config:load_custom_tokenizers()
+    if not success then
+      rspamd_logger.errx('cannot load custom tokenizers: %s', tokenizer_err or 'unknown error')
+      -- Don't exit here as custom tokenizers are optional
+      rspamd_logger.warnx('proceeding without custom tokenizers')
+    end
+  end
+end
+
+-- Helper function to ensure proper cleanup of tokenizers
+local function cleanup_tokenizers()
+  if rspamd_config then
+    rspamd_config:unload_custom_tokenizers()
+  end
 end
 
 local function load_task(opts, fname)
@@ -212,16 +299,63 @@ local function load_task(opts, fname)
     fname = '-'
   end
 
-  local res,task = rspamd_task.load_from_file(fname, rspamd_config)
+  local task = rspamd_task.create(rspamd_config, rspamadm_ev_base)
+  task:set_session(rspamadm_session)
+  task:set_resolver(rspamadm_dns_resolver)
 
-  if not res then
-    parser:error(string.format('cannot read message from %s: %s', fname,
-        task))
+  if opts.raw then
+    local f
+    if fname == '-' then
+      f = io.stdin
+    else
+      f = io.open(fname, "rb")
+    end
+
+    if not f then
+      parser:error("cannot open file " .. fname)
+    end
+
+    local content = f:read("*a")
+    if fname ~= '-' then
+      f:close()
+    end
+
+    local lua_magic = require "lua_magic"
+    local dummy_part = {
+      get_content = function()
+        return content
+      end,
+      get_filename = function()
+        return fname
+      end,
+    }
+    local _, type_data = lua_magic.detect(dummy_part, rspamd_config)
+    local ct = "application/octet-stream"
+    if type_data and type_data.type then
+      ct = type_data.type
+    end
+
+    if fname:match('%.pdf$') and (not ct or ct == 'application/octet-stream' or ct == 'binary') then
+      ct = 'application/pdf'
+    end
+
+    -- Construct message
+    local msg = string.format("Content-Type: %s\r\nContent-Transfer-Encoding: 8bit\r\n\r\n", ct)
+    task:load_from_string(msg .. content)
+  else
+    local res = task:load_from_file(fname)
+
+    if not res then
+      parser:error(string.format('cannot read message from %s: %s', fname,
+          task))
+      return nil
+    end
   end
 
   if not task:process_message() then
     parser:error(string.format('cannot read message from %s: %s', fname,
         'failed to parse'))
+    return nil
   end
 
   return task
@@ -237,19 +371,30 @@ local function maybe_print_fname(opts, fname)
   end
 end
 
+local function output_fmt(opts)
+  local fmt = 'json'
+  if opts.compact then
+    fmt = 'json-compact'
+  end
+  if opts.ucl then
+    fmt = 'ucl'
+  end
+  if opts.messagepack then
+    fmt = 'msgpack'
+  end
+
+  return fmt
+end
+
 -- Print elements in form
 -- filename -> table of elements
 local function print_elts(elts, opts, func)
   local fun = require "fun"
 
   if opts.json or opts.ucl then
-    local fmt = 'json'
-    if opts.compact then fmt = 'json-compact' end
-    if opts.ucl then fmt = 'ucl' end
-    io.write(ucl.to_format(elts, fmt))
+    io.write(ucl.to_format(elts, output_fmt(opts)))
   else
     fun.each(function(fname, elt)
-
       if not opts.json and not opts.ucl then
         if func then
           elt = fun.map(func, elt)
@@ -271,7 +416,7 @@ local function extract_handler(opts)
 
   if opts.words then
     -- Enable stemming and urls detection
-    load_config(opts)
+    load_config(opts, true) -- Load with custom tokenizers
     rspamd_url.init(rspamd_config:get_tld_path())
     rspamd_config:init_subsystem('langdet')
   end
@@ -287,33 +432,32 @@ local function extract_handler(opts)
       if not opts.json and not opts.ucl then
         table.insert(out,
             rspamd_logger.slog('Part: %s: %s, language: %s, size: %s (%s raw), words: %s',
-            part:get_mimepart():get_digest():sub(1,8),
-            t,
-            part:get_language(),
-            part:get_length(), part:get_raw_length(),
-            part:get_words_count()))
+                part:get_mimepart():get_digest():sub(1, 8),
+                t,
+                part:get_language(),
+                part:get_length(), part:get_raw_length(),
+                part:get_words_count()))
         table.insert(out,
             rspamd_logger.slog('Stats: %s',
-            fun.foldl(function(acc, k, v)
-              if acc ~= '' then
-                return string.format('%s, %s:%s', acc, k, v)
-              else
-                return string.format('%s:%s', k,v)
-              end
-            end, '', part:get_stats())))
+                fun.foldl(function(acc, k, v)
+                  if acc ~= '' then
+                    return string.format('%s, %s:%s', acc, k, v)
+                  else
+                    return string.format('%s:%s', k, v)
+                  end
+                end, '', part:get_stats())))
       end
     end
   end
 
   local function maybe_print_mime_part_info(part, out)
     if opts.part then
-
       if not opts.json and not opts.ucl then
-        local mtype,msubtype = part:get_type()
-        local det_mtype,det_msubtype = part:get_detected_type()
+        local mtype, msubtype = part:get_type()
+        local det_mtype, det_msubtype = part:get_detected_type()
         table.insert(out,
             rspamd_logger.slog('Mime Part: %s: %s/%s (%s/%s detected), filename: %s (%s detected ext), size: %s',
-                part:get_digest():sub(1,8),
+                part:get_digest():sub(1, 8),
                 mtype, msubtype,
                 det_mtype, det_msubtype,
                 part:get_filename(),
@@ -345,7 +489,7 @@ local function extract_handler(opts)
     end
   end
 
-  for _,fname in ipairs(opts.file) do
+  for _, fname in ipairs(opts.file) do
     local task = load_task(opts, fname)
     out_elts[fname] = {}
 
@@ -355,18 +499,44 @@ local function extract_handler(opts)
     end
 
     if opts.words then
-      local howw = opts['words_format'] or 'stem'
+      local how_words = opts['words_format'] or 'stem'
       table.insert(out_elts[fname], 'meta_words: ' ..
-          print_words(task:get_meta_words(howw), howw == 'full'))
+          print_words(task:get_meta_words(how_words), how_words == 'full'))
     end
 
     if opts.text or opts.html then
-      local mp = task:get_parts() or {}
+      local mp_all = task:get_parts(true) or {}
 
-      for _,mime_part in ipairs(mp) do
+      -- Build map: parent_part -> injected_text_part
+      local injected_map = {}
+      for _, p in ipairs(mp_all) do
+        if p:is_injected() and p:is_text() then
+
+          local parent = p:get_parent()
+          if parent then
+            injected_map[parent:get_digest()] = p:get_text()
+          end
+        end
+      end
+
+      -- Build table: {{part, injected_text or nil}, ...}
+      local parts_to_process = {}
+      for _, p in ipairs(mp_all) do
+        if not p:is_injected() then
+          table.insert(parts_to_process, { p, injected_map[p:get_digest()] })
+        end
+      end
+
+      -- Process the parts
+      for _, entry in ipairs(parts_to_process) do
+        local mime_part = entry[1]
+        local injected_part = entry[2]
         local how = opts.output
         local part
-        if mime_part:is_text() then part = mime_part:get_text() end
+
+        if mime_part:is_text() then
+          part = mime_part:get_text()
+        end
 
         if part and opts.text and not part:is_html() then
           maybe_print_text_part_info(part, out_elts[fname])
@@ -376,12 +546,28 @@ local function extract_handler(opts)
           end
 
           if opts.words then
-            local howw = opts['words_format'] or 'stem'
-            table.insert(out_elts[fname], print_words(part:get_words(howw),
-                howw == 'full'))
+            local how_words = opts['words_format'] or 'stem'
+            table.insert(out_elts[fname], print_words(part:get_words(how_words),
+                how_words == 'full'))
           else
             table.insert(out_elts[fname], tostring(part:get_content(how)))
           end
+        elseif injected_part and opts.text and not injected_part:is_html() then
+          -- Show parent part info but content from injected child
+          maybe_print_mime_part_info(mime_part, out_elts[fname])
+          if not opts.json and not opts.ucl then
+            table.insert(out_elts[fname], string.format('[Extracted text from %s]',
+                mime_part:get_filename() or 'attachment'))
+            table.insert(out_elts[fname], '\n')
+          end
+
+          if opts.words then
+            local how_words = opts['words_format'] or 'stem'
+            table.insert(out_elts[fname], print_words(injected_part:get_words(how_words),
+                how_words == 'full'))
+          else
+            table.insert(out_elts[fname], tostring(injected_part:get_content(how)))
+          end
         elseif part and opts.html and part:is_html() then
           maybe_print_text_part_info(part, out_elts[fname])
           maybe_print_mime_part_info(mime_part, out_elts[fname])
@@ -390,9 +576,9 @@ local function extract_handler(opts)
           end
 
           if opts.words then
-            local howw = opts['words_format'] or 'stem'
-            table.insert(out_elts[fname], print_words(part:get_words(howw),
-                howw == 'full'))
+            local how_words = opts['words_format'] or 'stem'
+            table.insert(out_elts[fname], print_words(part:get_words(how_words),
+                how_words == 'full'))
           else
             if opts.structure then
               local hc = part:get_html()
@@ -412,26 +598,32 @@ local function extract_handler(opts)
               end
 
               hc:foreach_tag('any', function(tag)
-                  local elt = {}
-                  local ex = tag:get_extra()
-                  elt.tag = tag:get_type()
-                  if ex then
-                    elt.extra = ex
-                  end
-                  local content = tag:get_content()
-                  if content then
-                    elt.content = tostring(content)
-                  end
-                  local style = tag:get_style()
-                  if style then
-                    elt.style = style
-                  end
-                  table.insert(res, elt)
+                local elt = {}
+                local ex = tag:get_extra()
+                elt.tag = tag:get_type()
+                if ex then
+                  elt.extra = ex
+                end
+                local content = tag:get_content()
+                if content then
+                  elt.content = tostring(content)
+                end
+                local style = tag:get_style()
+                if style then
+                  elt.style = style
+                end
+                table.insert(res, elt)
               end)
               table.insert(out_elts[fname], res)
-            else -- opts.structure
+            else
+              -- opts.structure
               table.insert(out_elts[fname], tostring(part:get_content(how)))
             end
+            if opts.invisible then
+              local hc = part:get_html()
+              table.insert(out_elts[fname], string.format('invisible content: %s',
+                  tostring(hc:get_invisible())))
+            end
           end
         end
 
@@ -447,20 +639,27 @@ local function extract_handler(opts)
 
   print_elts(out_elts, opts, process_func)
   -- To avoid use after free we postpone tasks destruction
-  for _,task in ipairs(tasks) do task:destroy() end
+  for _, task in ipairs(tasks) do
+    task:destroy()
+  end
+
+  -- Cleanup custom tokenizers if they were loaded
+  if opts.words then
+    cleanup_tokenizers()
+  end
 end
 
 local function stat_handler(opts)
   local fun = require "fun"
   local out_elts = {}
 
-  load_config(opts)
+  load_config(opts, true)                      -- Load with custom tokenizers for stat generation
   rspamd_url.init(rspamd_config:get_tld_path())
   rspamd_config:init_subsystem('langdet,stat') -- Needed to gen stat tokens
 
   local process_func
 
-  for _,fname in ipairs(opts.file) do
+  for _, fname in ipairs(opts.file) do
     local task = load_task(opts, fname)
     out_elts[fname] = {}
 
@@ -476,7 +675,9 @@ local function stat_handler(opts)
       process_func = function(e)
         return string.format('%s (%d): "%s"+"%s", [%s]', e.data, e.win, e.t1 or "",
             e.t2 or "", table.concat(fun.totable(
-                fun.map(function(k) return k end, e.flags)), ","))
+                fun.map(function(k)
+                  return k
+                end, e.flags)), ","))
       end
     elseif opts.fuzzy then
       local parts = task:get_parts() or {}
@@ -485,7 +686,7 @@ local function stat_handler(opts)
         local ret = string.format('part: %s(%s): %s', e.type, e.file or "", e.digest)
         if opts.shingles and e.shingles then
           local sgl = {}
-          for _,s in ipairs(e.shingles) do
+          for _, s in ipairs(e.shingles) do
             table.insert(sgl, string.format('%s: %s+%s+%s', s[1], s[2], s[3], s[4]))
           end
 
@@ -493,26 +694,26 @@ local function stat_handler(opts)
         end
         return ret
       end
-      for _,part in ipairs(parts) do
+      for _, part in ipairs(parts) do
         if not part:is_multipart() then
           local text = part:get_text()
 
           if text then
-            local digest,shingles = text:get_fuzzy_hashes(task:get_mempool())
+            local digest, shingles = text:get_fuzzy_hashes(task:get_mempool())
             table.insert(out_elts[fname], {
               digest = digest,
               shingles = shingles,
               type = string.format('%s/%s',
-                  ({part:get_type()})[1],
-                  ({part:get_type()})[2])
+                  ({ part:get_type() })[1],
+                  ({ part:get_type() })[2])
             })
           else
             table.insert(out_elts[fname], {
               digest = part:get_digest(),
               file = part:get_filename(),
               type = string.format('%s/%s',
-                  ({part:get_type()})[1],
-                  ({part:get_type()})[2])
+                  ({ part:get_type() })[1],
+                  ({ part:get_type() })[2])
             })
           end
         end
@@ -523,16 +724,21 @@ local function stat_handler(opts)
   end
 
   print_elts(out_elts, opts, process_func)
+
+  -- Cleanup custom tokenizers
+  cleanup_tokenizers()
 end
 
 local function urls_handler(opts)
-  load_config(opts)
+  load_config(opts, false) -- URLs don't need custom tokenizers
   rspamd_url.init(rspamd_config:get_tld_path())
   local out_elts = {}
 
-  if opts.json then rspamd_logger.messagex('[') end
+  if opts.json then
+    rspamd_logger.messagex('[')
+  end
 
-  for _,fname in ipairs(opts.file) do
+  for _, fname in ipairs(opts.file) do
     out_elts[fname] = {}
     local task = load_task(opts, fname)
     local elts = {}
@@ -567,7 +773,7 @@ local function urls_handler(opts)
       end
     end
 
-    for _,u in ipairs(task:get_urls(true)) do
+    for _, u in ipairs(task:get_urls(true)) do
       process_url(u)
     end
 
@@ -634,12 +840,11 @@ local function urls_handler(opts)
         end
       end
 
-
-      for s,u in lua_util.spairs(elts, sfunc) do
+      for s, u in lua_util.spairs(elts, sfunc) do
         process_elt(s, u)
       end
     else
-      for s,u in pairs(elts) do
+      for s, u in pairs(elts) do
         process_elt(s, u)
       end
     end
@@ -665,7 +870,7 @@ local function newline(task)
 end
 
 local function modify_handler(opts)
-  load_config(opts)
+  load_config(opts, false) -- Modification doesn't need custom tokenizers
   rspamd_url.init(rspamd_config:get_tld_path())
 
   local function read_file(file)
@@ -685,7 +890,7 @@ local function modify_handler(opts)
     html_footer = read_file(opts['html_footer'])
   end
 
-  for _,fname in ipairs(opts.file) do
+  for _, fname in ipairs(opts.file) do
     local task = load_task(opts, fname)
     local newline_s = newline(task)
     local seen_cte
@@ -694,14 +899,14 @@ local function modify_handler(opts)
     local out = {} -- Start with headers
 
     local function process_headers_cb(name, hdr)
-      for _,h in ipairs(opts['remove_header']) do
+      for _, h in ipairs(opts['remove_header']) do
         if name:match(h) then
           return
         end
       end
 
-      for _,h in ipairs(opts['rewrite_header']) do
-        local hname,hpattern = h:match('^([^=]+)=(.+)$')
+      for _, h in ipairs(opts['rewrite_header']) do
+        local hname, hpattern = h:match('^([^=]+)=(.+)$')
         if hname == name then
           local new_value = string.format(hpattern, hdr.decoded)
           new_value = string.format('%s:%s%s',
@@ -731,10 +936,10 @@ local function modify_handler(opts)
       out[#out + 1] = hdr.raw:gsub('\r?\n?$', '')
     end
 
-    task:headers_foreach(process_headers_cb, {full = true})
+    task:headers_foreach(process_headers_cb, { full = true })
 
-    for _,h in ipairs(opts['add_header']) do
-      local hname,hvalue = h:match('^([^=]+)=(.+)$')
+    for _, h in ipairs(opts['add_header']) do
+      local hname, hvalue = h:match('^([^=]+)=(.+)$')
 
       if hname and hvalue then
         out[#out + 1] = string.format('%s: %s', hname,
@@ -751,14 +956,14 @@ local function modify_handler(opts)
     out[#out + 1] = ''
 
     if rewrite.out then
-      for _,o in ipairs(rewrite.out) do
+      for _, o in ipairs(rewrite.out) do
         out[#out + 1] = o
       end
     else
-      out[#out + 1] = {task:get_rawbody(), false}
+      out[#out + 1] = { task:get_rawbody(), false }
     end
 
-    for _,o in ipairs(out) do
+    for _, o in ipairs(out) do
       if type(o) == 'string' then
         io.write(o)
         io.write(newline_s)
@@ -784,7 +989,7 @@ local function modify_handler(opts)
 end
 
 local function sign_handler(opts)
-  load_config(opts)
+  load_config(opts, false) -- Signing doesn't need custom tokenizers
   rspamd_url.init(rspamd_config:get_tld_path())
 
   local lua_dkim = require("lua_ffi").dkim
@@ -806,7 +1011,7 @@ local function sign_handler(opts)
     os.exit(1)
   end
 
-  for _,fname in ipairs(opts.file) do
+  for _, fname in ipairs(opts.file) do
     local task = load_task(opts, fname)
     local ctx = lua_dkim.create_sign_context(task, sign_key, nil, opts.algorithm)
 
@@ -842,13 +1047,193 @@ local function sign_handler(opts)
   end
 end
 
+local function strip_handler(opts)
+  load_config(opts, false) -- Stripping doesn't need custom tokenizers
+  rspamd_url.init(rspamd_config:get_tld_path())
+
+  for _, fname in ipairs(opts.file) do
+    local task = load_task(opts, fname)
+    local newline_s = newline(task)
+
+    local rewrite = lua_mime.remove_attachments(task, {
+      keep_images = opts.keep_images,
+      min_text_size = opts.min_text_size,
+      max_text_size = opts.max_text_size
+    }) or {}
+    local out = {} -- Start with headers
+
+    local function process_headers_cb(name, hdr)
+      out[#out + 1] = hdr.raw:gsub('\r?\n?$', '')
+    end
+
+    task:headers_foreach(process_headers_cb, { full = true })
+    -- End of headers
+    out[#out + 1] = ''
+
+    if rewrite.out then
+      for _, o in ipairs(rewrite.out) do
+        out[#out + 1] = o
+      end
+    else
+      out[#out + 1] = { task:get_rawbody(), false }
+    end
+
+    for _, o in ipairs(out) do
+      if type(o) == 'string' then
+        io.write(o)
+        io.write(newline_s)
+      elseif type(o) == 'table' then
+        io.flush()
+        if type(o[1]) == 'string' then
+          io.write(o[1])
+        else
+          o[1]:save_in_file(1)
+        end
+
+        if o[2] then
+          io.write(newline_s)
+        end
+      else
+        o:save_in_file(1)
+        io.write(newline_s)
+      end
+    end
+
+    task:destroy() -- No automatic dtor
+  end
+end
+
+local function anonymize_handler(opts)
+  load_config(opts, false) -- Anonymization doesn't need custom tokenizers
+  rspamd_url.init(rspamd_config:get_tld_path())
+
+  for _, fname in ipairs(opts.file) do
+    local task = load_task(opts, fname)
+    local newline_s = newline(task)
+
+    local rewrite = lua_mime.anonymize_message(task, opts)
+
+    if not rewrite or not rewrite.out then
+      rspamd_logger.errx('cannot anonymize message from %s', fname)
+      task:destroy()
+      os.exit(1)
+    end
+
+    for _, o in ipairs(rewrite.out) do
+      if type(o) == 'string' then
+        io.write(o)
+        io.write(newline_s)
+      elseif type(o) == 'table' then
+        io.flush()
+        if type(o[1]) == 'string' then
+          io.write(o[1])
+        else
+          o[1]:save_in_file(1)
+        end
+
+        if o[2] then
+          io.write(newline_s)
+        end
+      else
+        o:save_in_file(1)
+        io.write(newline_s)
+      end
+    end
+
+    task:destroy() -- No automatic dtor
+  end
+end
+
+-- Strips directories and .extensions (if present) from a filepath
+local function filename_only(filepath)
+  local filename = filepath:match(".*%/([^%.]+)")
+  if not filename then
+    filename = filepath:match("([^%.]+)")
+  end
+  return filename
+end
+
+assert(filename_only("very_simple") == "very_simple")
+assert(filename_only("/home/very_simple.eml") == "very_simple")
+assert(filename_only("very_simple.eml") == "very_simple")
+assert(filename_only("very_simple.example.eml") == "very_simple")
+assert(filename_only("/home/very_simple") == "very_simple")
+assert(filename_only("home/very_simple") == "very_simple")
+assert(filename_only("./home/very_simple") == "very_simple")
+assert(filename_only("../home/very_simple.eml") == "very_simple")
+assert(filename_only("/home/dir.with.dots/very_simple.eml") == "very_simple")
+
+--Write the dump content to file or standard out
+local function write_dump_content(dump_content, fname, extension, outdir)
+  if type(dump_content) == "string" then
+    dump_content = rspamd_text.fromstring(dump_content)
+  end
+
+  local wrote_filepath = nil
+  if outdir then
+    if outdir:sub(-1) ~= "/" then
+      outdir = outdir .. "/"
+    end
+
+    local outpath = string.format("%s%s.%s", outdir, filename_only(fname), extension)
+    if rspamd_util.file_exists(outpath) then
+      os.remove(outpath)
+    end
+    if dump_content:save_in_file(outpath) then
+      wrote_filepath = outpath
+      io.write(wrote_filepath .. "\n")
+    else
+      io.stderr:write(string.format("Unable to save dump content to file: %s\n", outpath))
+    end
+  else
+    dump_content:save_in_file(1)
+  end
+  return wrote_filepath
+end
+
+-- Get the formatted ucl (split or unsplit) or the raw task content
+local function get_dump_content(task, opts, fname)
+  if opts.ucl or opts.json or opts.messagepack then
+    local ucl_object = lua_mime.message_to_ucl(task)
+    -- Split out the content field into separate raws and update the ucl
+    if opts.split then
+      for i, part in ipairs(ucl_object.parts) do
+        if part.content then
+          local part_filename = string.format("%s-part%d", filename_only(fname), i)
+          local part_path = write_dump_content(part.content, part_filename, "raw", opts.outdir)
+          if part_path then
+            part.content = ucl.null
+            part.content_path = part_path
+          end
+        end
+      end
+    end
+    local extension = output_fmt(opts)
+    return ucl.to_format(ucl_object, extension), extension
+  end
+  return task:get_content(), "mime"
+end
+
+local function dump_handler(opts)
+  load_config(opts, false) -- Dumping doesn't need custom tokenizers
+  rspamd_url.init(rspamd_config:get_tld_path())
+
+  for _, fname in ipairs(opts.file) do
+    local task = load_task(opts, fname)
+    local data, extension = get_dump_content(task, opts, fname)
+    write_dump_content(data, fname, extension, opts.outdir)
+
+    task:destroy() -- No automatic dtor
+  end
+end
+
 local function handler(args)
   local opts = parser:parse(args)
 
   local command = opts.command
 
   if type(opts.file) == 'string' then
-    opts.file = {opts.file}
+    opts.file = { opts.file }
   elseif type(opts.file) == 'none' then
     opts.file = {}
   end
@@ -861,8 +1246,14 @@ local function handler(args)
     urls_handler(opts)
   elseif command == 'modify' then
     modify_handler(opts)
+  elseif command == 'strip' then
+    strip_handler(opts)
   elseif command == 'sign' then
     sign_handler(opts)
+  elseif command == 'dump' then
+    dump_handler(opts)
+  elseif command == 'anonymize' then
+    anonymize_handler(opts)
   else
     parser:error('command %s is not implemented', command)
   end
@@ -870,7 +1261,7 @@ end
 
 return {
   name = 'mime',
-  aliases = {'mime_tool'},
+  aliases = { 'mime_tool' },
   handler = handler,
   description = parser._description
-}
\ No newline at end of file
+}
