@@ -62,6 +62,49 @@ if confighelp then
   reason_header = "X-GPT-Reason";
   # Use JSON format for response
   json = false;
+  # Optional: pass request timeout to the server (in seconds)
+  # WARNING: Not all API implementations support this parameter (e.g., standard OpenAI API doesn't)
+  # Only enable if your API endpoint/proxy specifically supports max_completion_time parameter
+  # If not set, this parameter will not be sent to the server
+  # Note: the actual value sent to server is multiplied by 0.95 to account for
+  # connection setup, SSL handshake, and data transfer overhead
+  # request_timeout = 8;
+
+  # Optional user/domain context in Redis
+  context = {
+    enabled = true; # fetch and inject user/domain conversation context
+    # scope level for identity: user | domain | esld
+    level = "user";
+    # redis key structure: <key_prefix>:<identity>:<key_suffix>
+    key_prefix = "user";
+    key_suffix = "mail_context";
+    # sliding window and TTLs
+    max_messages = 40; # keep up to N compact message summaries
+    min_messages = 5; # warm-up: inject context only after N messages collected
+    message_ttl = 14d; # forget messages older than this when recomputing
+    ttl = 30d; # Redis key TTL
+    top_senders = 5; # track top senders
+    summary_max_chars = 512; # compress body to this size for storage
+    flagged_phrases = ["reset your password", "click here to verify"]; # optional list
+    last_labels_count = 10; # keep last N labels
+    as_system = true; # place context snippet as additional system message
+  };
+
+  # Optional web search context (extract domains from URLs and search for context)
+  search_context = {
+    enabled = false; # fetch web search context for domains in email
+    search_url = "https://leta.mullvad.net/search/__data.json"; # Search API endpoint
+    search_engine = "brave"; # Search engine (brave, google, etc.)
+    max_domains = 3; # Maximum domains to search
+    max_results_per_query = 3; # Maximum results per domain
+    timeout = 5; # HTTP timeout in seconds
+    cache_ttl = 3600; # Cache TTL in seconds (1 hour)
+    cache_key_prefix = "gpt_search"; # Redis cache key prefix
+    as_system = true; # Inject as system message (false = user message)
+    # Optional gating expressions to enable/disable search context dynamically
+    # enable_expression = { ... }; # Enable for specific conditions
+    # disable_expression = { ... }; # Disable for specific conditions
+  };
   }
   ]])
   return
@@ -76,6 +119,11 @@ local lua_redis = require "lua_redis"
 local ucl = require "ucl"
 -- local fun = require "fun" -- no longer needed after llm_common usage
 local lua_cache = require "lua_cache"
+local llm_context = require "llm_context"
+local llm_search_context = require "llm_search_context"
+local lua_maps_expressions = require "lua_maps_expressions"
+local lua_maps = require "lua_maps"
+local lua_selectors = require "lua_selectors"
 
 -- Exclude checks if one of those is found
 local default_symbols_to_except = {
@@ -109,6 +157,11 @@ local default_extra_symbols = {
     description = 'GPT model detected malware content',
     category = 'malware',
   },
+  GPT_UNCERTAIN = {
+    score = 0.0,
+    description = 'GPT model was uncertain about classification',
+    category = 'uncertain',
+  },
 }
 
 -- Should be filled from extra symbols
@@ -148,9 +201,135 @@ local settings = {
   json = false,
   extra_symbols = nil,
   cache_prefix = REDIS_PREFIX,
+  request_timeout = nil, -- Optional: pass request timeout to server (in seconds)
+  -- user/domain context options (nested table forwarded to llm_context)
+  context = {
+    enabled = false,
+    level = 'user', -- 'user' | 'domain' | 'esld'
+    key_prefix = 'user',
+    key_suffix = 'mail_context',
+    max_messages = 40,
+    min_messages = 5,      -- warm-up threshold: minimum messages before injecting context into prompt
+    message_ttl = 1209600, -- 14d
+    ttl = 2592000,         -- 30d
+    top_senders = 5,
+    summary_max_chars = 512,
+    flagged_phrases = { 'reset your password', 'click here to verify' },
+    last_labels_count = 10,
+    as_system = true, -- inject context snippet as system message; false => user message
+    -- Optional gating using selectors and maps to enable/disable context dynamically
+    -- One can use either a simple enable_map or a full maps expression
+    -- Example enable_map:
+    -- enable_map = { selector = "esld_principal_recipient_domain", map = "/etc/rspamd/context-enabled-domains.map", type = "set" }
+    enable_map = nil,
+    -- Example enable_expression:
+    -- enable_expression = {
+    --   rules = {
+    --     dom = { selector = "esld_principal_recipient_domain", map = "/etc/rspamd/context-enabled-domains.map" },
+    --     user = { selector = "user", map = "/etc/rspamd/context-enabled-users.map" },
+    --   },
+    --   expression = "dom | user"
+    -- }
+    enable_expression = nil,
+    -- Optional negative gating
+    disable_expression = nil,
+  },
+  -- Web search context options (for extracting and searching domains from URLs)
+  search_context = {
+    enabled = false,
+    search_url = 'https://leta.mullvad.net/search/__data.json', -- Search API endpoint
+    search_engine = 'brave',                            -- Search engine (brave, google, etc.)
+    max_domains = 3,                                    -- Maximum domains to search
+    max_results_per_query = 3,                          -- Maximum results per domain
+    timeout = 5,                                        -- HTTP timeout in seconds
+    cache_ttl = 3600,                                   -- Cache TTL (1 hour)
+    cache_key_prefix = 'gpt_search',                    -- Redis cache key prefix
+    as_system = true,                                   -- Inject as system message (false = user message)
+    -- Optional gating using selectors and maps to enable/disable search context dynamically
+    enable_expression = nil,
+    disable_expression = nil,
+  },
 }
 local redis_params
 local cache_context
+local compiled_context_gating = {
+  enable_expr = nil,
+  disable_expr = nil,
+  enable_map = nil, -- { selector_fn, map }
+}
+local compiled_search_context_gating = {
+  enable_expr = nil,
+  disable_expr = nil,
+}
+
+local function is_context_enabled_for_task(task)
+  local ctx = settings.context
+  if not ctx then return false end
+
+  local enabled = ctx.enabled or false
+
+  -- Positive gating via expression
+  if compiled_context_gating.enable_expr then
+    local res = compiled_context_gating.enable_expr:process(task)
+    if res then
+      enabled = true
+    end
+  end
+
+  -- Positive gating via simple map
+  if compiled_context_gating.enable_map then
+    local vals = compiled_context_gating.enable_map.selector_fn(task)
+    local matched = false
+    if type(vals) == 'table' then
+      for _, v in ipairs(vals) do
+        if compiled_context_gating.enable_map.map:get_key(v) then
+          matched = true
+          break
+        end
+      end
+    elseif vals then
+      matched = compiled_context_gating.enable_map.map:get_key(vals) and true or false
+    end
+    if matched then
+      enabled = true
+    end
+  end
+
+  -- Negative gating
+  if enabled and compiled_context_gating.disable_expr then
+    local res = compiled_context_gating.disable_expr:process(task)
+    if res then
+      enabled = false
+    end
+  end
+
+  return enabled
+end
+
+local function is_search_context_enabled_for_task(task)
+  local ctx = settings.search_context
+  if not ctx then return false end
+
+  local enabled = ctx.enabled or false
+
+  -- Positive gating via expression
+  if compiled_search_context_gating.enable_expr then
+    local res = compiled_search_context_gating.enable_expr:process(task)
+    if res then
+      enabled = true
+    end
+  end
+
+  -- Negative gating
+  if enabled and compiled_search_context_gating.disable_expr then
+    local res = compiled_search_context_gating.disable_expr:process(task)
+    if res then
+      enabled = false
+    end
+  end
+
+  return enabled
+end
 
 local function default_condition(task)
   -- Check result
@@ -327,12 +506,12 @@ local function default_openai_json_conversion(task, input)
       elseif reply.probability == "low" then
         spam_score = 0.1
       else
-        rspamd_logger.infox(task, "cannot convert to spam probability: %s", reply.probability)
+        lua_util.debugm(N, task, "cannot convert to spam probability: %s", reply.probability)
       end
     end
 
     if type(reply.usage) == 'table' then
-      rspamd_logger.infox(task, 'usage: %s tokens', reply.usage.total_tokens)
+      lua_util.debugm(N, task, 'usage: %s tokens', reply.usage.total_tokens)
     end
 
     return spam_score, reply.reason, {}
@@ -370,9 +549,22 @@ local function default_openai_plain_conversion(task, input)
   end
 
   local first_message = reply.choices[1].message.content
+  local finish_reason = reply.choices[1].finish_reason or 'unknown'
 
   if not first_message or first_message == "" then
-    rspamd_logger.errx(task, 'no content in the first message')
+    if finish_reason == 'length' then
+      -- Token limit exceeded - provide helpful error message
+      local usage = reply.usage or {}
+      local completion_tokens = usage.completion_tokens or 0
+      local reasoning_tokens = usage.completion_tokens_details and usage.completion_tokens_details.reasoning_tokens or 0
+      rspamd_logger.errx(task, 'LLM response truncated: token limit exceeded. ' ..
+        'Used %s completion tokens (including %s reasoning tokens). ' ..
+        'Increase max_completion_tokens in model_parameters config for this model.',
+        completion_tokens, reasoning_tokens)
+    else
+      rspamd_logger.errx(task, 'no content in the first message (finish_reason: %s, usage: %s)',
+        finish_reason, reply.usage and ucl.to_format(reply.usage, 'json-compact') or 'none')
+    end
     return
   end
 
@@ -386,7 +578,7 @@ local function default_openai_plain_conversion(task, input)
   local categories = lua_util.str_split(clean_reply_line(lines[3]), ',')
 
   if type(reply.usage) == 'table' then
-    rspamd_logger.infox(task, 'usage: %s tokens', reply.usage.total_tokens)
+    lua_util.debugm(N, task, 'usage: %s tokens', reply.usage.total_tokens)
   end
 
   if spam_score then
@@ -487,12 +679,12 @@ local function default_ollama_json_conversion(task, input)
       elseif reply.probability == "low" then
         spam_score = 0.1
       else
-        rspamd_logger.infox(task, "cannot convert to spam probability: %s", reply.probability)
+        lua_util.debugm(N, task, "cannot convert to spam probability: %s", reply.probability)
       end
     end
 
     if type(reply.usage) == 'table' then
-      rspamd_logger.infox(task, 'usage: %s tokens', reply.usage.total_tokens)
+      lua_util.debugm(N, task, 'usage: %s tokens', reply.usage.total_tokens)
     end
 
     return spam_score, reply.reason, {}
@@ -542,7 +734,7 @@ local function insert_results(task, result, sel_part)
     if result.categories then
       process_categories(task, result.categories)
     end
-  else
+  elseif result.probability < 0.5 then
     task:insert_result('GPT_HAM', (0.5 - result.probability) * 2, tostring(result.probability))
     if settings.autolearn then
       task:set_flag("learn_ham")
@@ -550,17 +742,37 @@ local function insert_results(task, result, sel_part)
     if result.categories then
       process_categories(task, result.categories)
     end
+  else
+    -- probability == 0.5, uncertain result, don't set GPT_SPAM/GPT_HAM
+    if result.categories then
+      process_categories(task, result.categories)
+    end
   end
   if result.reason and settings.reason_header then
-    local v = lua_util.fold_header_with_encoding(task, settings.reason_header,
-      tostring(result.reason), { encode = 'auto' })
-    lua_mime.modify_headers(task,
-      { add = { [settings.reason_header] = { value = v, order = 1 } } })
+    if type(settings.reason_header) == 'string' and #result.reason > 0 then
+      local ok, v = pcall(lua_util.fold_header_with_encoding, task, settings.reason_header,
+        result.reason, { encode = false, structured = false })
+      if ok and v then
+        lua_mime.modify_headers(task,
+          { add = { [settings.reason_header] = { value = v, order = 1 } } })
+      else
+        rspamd_logger.warnx(task, 'cannot fold header %s: %s; using raw value', settings.reason_header,
+          v)
+        -- Fallback: use raw value without encoding
+        lua_mime.modify_headers(task,
+          { add = { [settings.reason_header] = { value = result.reason, order = 1 } } })
+      end
+    end
   end
 
   if cache_context then
     lua_cache.cache_set(task, redis_cache_key(sel_part), result, cache_context)
   end
+
+  -- Update long-term user/domain context after classification
+  if redis_params and settings.context then
+    llm_context.update_after_classification(task, redis_params, settings.context, result, sel_part, N)
+  end
 end
 
 local function check_consensus_and_insert_results(task, results, sel_part)
@@ -571,21 +783,21 @@ local function check_consensus_and_insert_results(task, results, sel_part)
   end
 
   local nspam, nham = 0, 0
-  local max_spam_prob, max_ham_prob = 0, 0
+  local max_spam_prob, max_ham_prob = 0, 1.0
   local reasons = {}
 
   for _, result in ipairs(results) do
-    if result.success then
+    if result.success and result.probability then
       if result.probability > 0.5 then
         nspam = nspam + 1
         max_spam_prob = math.max(max_spam_prob, result.probability)
         lua_util.debugm(N, task, "model: %s; spam: %s; reason: '%s'",
-          result.model, result.probability, result.reason)
+          result.model or 'unknown', result.probability, result.reason or 'no reason')
       else
         nham = nham + 1
         max_ham_prob = math.min(max_ham_prob, result.probability)
         lua_util.debugm(N, task, "model: %s; ham: %s; reason: '%s'",
-          result.model, result.probability, result.reason)
+          result.model or 'unknown', result.probability, result.reason or 'no reason')
       end
 
       if result.reason then
@@ -595,39 +807,53 @@ local function check_consensus_and_insert_results(task, results, sel_part)
   end
 
   lua_util.shuffle(reasons)
-  local reason = reasons[1] or nil
+  local reason_obj = reasons[1]
+  local reason_text = reason_obj and reason_obj.reason or nil
+  local reason_categories = reason_obj and reason_obj.categories or nil
 
   if nspam > nham and max_spam_prob > 0.75 then
     insert_results(task, {
         probability = max_spam_prob,
-        reason = reason.reason,
-        categories = reason.categories,
+        reason = reason_text,
+        categories = reason_categories,
       },
       sel_part)
   elseif nham > nspam and max_ham_prob < 0.25 then
     insert_results(task, {
         probability = max_ham_prob,
-        reason = reason.reason,
-        categories = reason.categories,
+        reason = reason_text,
+        categories = reason_categories,
       },
       sel_part)
   else
-    -- No consensus
-    lua_util.debugm(N, task, "no consensus")
+    -- No consensus - still cache and set uncertain symbol to avoid re-querying LLM
+    lua_util.debugm(N, task, "no consensus: nspam=%s, nham=%s, max_spam_prob=%s, max_ham_prob=%s",
+      nspam, nham, max_spam_prob, max_ham_prob)
+    -- Use 0.5 (neutral) probability with uncertain marker
+    local uncertain_reason = reason_text or string.format(
+      "Uncertain classification: spam votes=%d (max %.2f), ham votes=%d (min %.2f)",
+      nspam, max_spam_prob, nham, max_ham_prob)
+    insert_results(task, {
+        probability = 0.5,
+        reason = uncertain_reason,
+        categories = { 'uncertain' },
+      },
+      sel_part)
+    task:insert_result('GPT_UNCERTAIN', 1.0)
   end
 end
 
 -- get_meta_llm_content moved to llm_common
 
-local function check_llm_uncached(task, content, sel_part)
-  return settings.specific_check(task, content, sel_part)
+local function check_llm_uncached(task, content, sel_part, context_snippet)
+  return settings.specific_check(task, content, sel_part, context_snippet)
 end
 
-local function check_llm_cached(task, content, sel_part)
+local function check_llm_cached(task, content, sel_part, context_snippet)
   local cache_key = redis_cache_key(sel_part)
 
   lua_cache.cache_get(task, cache_key, cache_context, settings.timeout * 1.5, function()
-    check_llm_uncached(task, content, sel_part)
+    check_llm_uncached(task, content, sel_part, context_snippet)
   end, function(_, err, data)
     if err then
       rspamd_logger.errx(task, 'cannot get cache: %s', err)
@@ -635,16 +861,21 @@ local function check_llm_cached(task, content, sel_part)
     end
 
     if data then
-      rspamd_logger.infox(task, 'found cached response %s', cache_key)
+      lua_util.debugm(N, task, 'found cached response %s', cache_key)
       insert_results(task, data, sel_part)
     else
-      check_llm_uncached(task, content, sel_part)
+      check_llm_uncached(task, content, sel_part, context_snippet)
     end
   end)
 end
 
-local function openai_check(task, content, sel_part)
+local function openai_check(task, content, sel_part, context_snippet)
   lua_util.debugm(N, task, "sending content to gpt: %s", content)
+  if context_snippet then
+    lua_util.debugm(N, task, "with context snippet (%s chars): %s", #context_snippet, context_snippet)
+  else
+    lua_util.debugm(N, task, "no context snippet")
+  end
 
   local upstream
   local results = {}
@@ -684,7 +915,7 @@ local function openai_check(task, content, sel_part)
     end
   end
 
-  -- Build messages exactly as in the original code if structured table provided
+  -- Build messages with optional user/domain context
   local user_messages
   if type(content) == 'table' then
     local subject_line = 'Subject: ' .. (content.subject or '')
@@ -700,22 +931,25 @@ local function openai_check(task, content, sel_part)
     }
   end
 
-  local body_base = {
-    stream = false,
-    messages = {
-      {
-        role = 'system',
-        content = settings.prompt
-      },
-      lua_util.unpack(user_messages)
-    }
+  local sys_messages = {
+    { role = 'system', content = settings.prompt }
   }
-
-  if type(settings.model) == 'string' then
-    settings.model = { settings.model }
+  if context_snippet and settings.context and settings.context.as_system ~= false then
+    table.insert(sys_messages, { role = 'system', content = context_snippet })
+  elseif context_snippet and settings.context and settings.context.as_system == false then
+    table.insert(user_messages, 1, { role = 'user', content = context_snippet })
   end
 
-  for idx, model in ipairs(settings.model) do
+  local body_base = {
+    stream = false,
+    messages = {}
+  }
+  for _, m in ipairs(sys_messages) do table.insert(body_base.messages, m) end
+  for _, m in ipairs(user_messages) do table.insert(body_base.messages, m) end
+
+  local models_list = type(settings.model) == 'string' and { settings.model } or settings.model
+
+  for idx, model in ipairs(models_list) do
     results[idx] = {
       success = false,
       checked = false
@@ -736,6 +970,13 @@ local function openai_check(task, content, sel_part)
       body.response_format = { type = "json_object" }
     end
 
+    -- Optionally add request timeout for server-side timeout control
+    -- Only pass if explicitly configured (not all API implementations support this)
+    -- Multiply by 0.95 to account for connection setup, SSL handshake, and data transfer time
+    if settings.request_timeout then
+      body.max_completion_time = settings.request_timeout * 0.95
+    end
+
     body.model = model
 
     upstream = settings.upstreams:get_upstream_round_robin()
@@ -766,8 +1007,13 @@ local function openai_check(task, content, sel_part)
   end
 end
 
-local function ollama_check(task, content, sel_part)
+local function ollama_check(task, content, sel_part, context_snippet)
   lua_util.debugm(N, task, "sending content to gpt: %s", content)
+  if context_snippet then
+    lua_util.debugm(N, task, "with context snippet (%s chars): %s", #context_snippet, context_snippet)
+  else
+    lua_util.debugm(N, task, "no context snippet")
+  end
 
   local upstream
   local results = {}
@@ -821,26 +1067,25 @@ local function ollama_check(task, content, sel_part)
     }
   end
 
-  if type(settings.model) == 'string' then
-    settings.model = { settings.model }
+  local models_list = type(settings.model) == 'string' and { settings.model } or settings.model
+
+  local sys_messages = {
+    { role = 'system', content = settings.prompt }
+  }
+  if context_snippet and settings.context and settings.context.as_system ~= false then
+    table.insert(sys_messages, { role = 'system', content = context_snippet })
+  elseif context_snippet and settings.context and settings.context.as_system == false then
+    table.insert(user_messages, 1, { role = 'user', content = context_snippet })
   end
 
   local body_base = {
     stream = false,
-    model = settings.model,
-    -- should not in body_base
-    -- max_tokens = settings.max_tokens,
-    -- temperature = settings.temperature,
-    messages = {
-      {
-        role = 'system',
-        content = settings.prompt
-      },
-      table.unpack(user_messages)
-    }
+    messages = {}
   }
+  for _, m in ipairs(sys_messages) do table.insert(body_base.messages, m) end
+  for _, m in ipairs(user_messages) do table.insert(body_base.messages, m) end
 
-  for idx, model in ipairs(settings.model) do
+  for idx, model in ipairs(models_list) do
     results[idx] = {
       success = false,
       checked = false
@@ -861,6 +1106,13 @@ local function ollama_check(task, content, sel_part)
       body.response_format = { type = "json_object" }
     end
 
+    -- Optionally add request timeout for server-side timeout control
+    -- Only pass if explicitly configured (not all API implementations support this)
+    -- Multiply by 0.95 to account for connection setup, SSL handshake, and data transfer time
+    if settings.request_timeout then
+      body.max_completion_time = settings.request_timeout * 0.95
+    end
+
     body.model = model
 
     upstream = settings.upstreams:get_upstream_round_robin()
@@ -891,8 +1143,33 @@ end
 local function gpt_check(task)
   local ret, content, sel_part = settings.condition(task)
 
+  -- Always update context if enabled, even when condition is not met
+  local context_enabled = redis_params and settings.context and is_context_enabled_for_task(task)
+  if context_enabled and not ret then
+    -- Condition not met (e.g. BAYES_SPAM, passthrough, etc.)
+    -- Update context without LLM call; infer result from task metrics
+    if not sel_part then
+      -- Try to get text part for context update
+      sel_part = lua_mime.get_displayed_text_part(task)
+    end
+    if sel_part then
+      local result = task:get_metric_result()
+      local inferred_result = nil
+      if result then
+        if result.action == 'reject' or (result.score and result.score > 10) then
+          inferred_result = { probability = 0.9, reason = 'rejected by filters', categories = {} }
+        elseif result.action == 'no action' and result.score and result.score < 0 then
+          inferred_result = { probability = 0.1, reason = 'ham by filters', categories = {} }
+        end
+      end
+      llm_context.update_after_classification(task, redis_params, settings.context, inferred_result, sel_part, N)
+    end
+    lua_util.debugm(N, task, "skip checking gpt as the condition is not met: %s; context updated", content)
+    return
+  end
+
   if not ret then
-    rspamd_logger.info(task, "skip checking gpt as the condition is not met: %s", content)
+    lua_util.debugm(N, task, "skip checking gpt as the condition is not met: %s", content)
     return
   end
 
@@ -901,11 +1178,62 @@ local function gpt_check(task)
     return
   end
 
-  if sel_part then
-    -- Check digest
-    check_llm_cached(task, content, sel_part)
+  local function proceed(combined_context)
+    if sel_part then
+      -- Check digest
+      check_llm_cached(task, content, sel_part, combined_context)
+    else
+      check_llm_uncached(task, content, nil, combined_context)
+    end
+  end
+
+  -- Check if we need to fetch search context
+  local search_context_enabled = is_search_context_enabled_for_task(task)
+
+  if context_enabled or search_context_enabled then
+    local pending_fetches = 0
+    local user_context_snippet = nil
+    local search_context_snippet = nil
+
+    local function maybe_proceed()
+      if pending_fetches == 0 then
+        -- Combine contexts
+        local combined_context = nil
+        if user_context_snippet and search_context_snippet then
+          combined_context = user_context_snippet .. "\n\n" .. search_context_snippet
+        elseif user_context_snippet then
+          combined_context = user_context_snippet
+        elseif search_context_snippet then
+          combined_context = search_context_snippet
+        end
+        proceed(combined_context)
+      end
+    end
+
+    if context_enabled then
+      pending_fetches = pending_fetches + 1
+      llm_context.fetch(task, redis_params, settings.context, function(_, _, snippet)
+        user_context_snippet = snippet
+        pending_fetches = pending_fetches - 1
+        maybe_proceed()
+      end, N)
+    end
+
+    if search_context_enabled then
+      pending_fetches = pending_fetches + 1
+      llm_search_context.fetch_and_format(task, redis_params, settings.search_context, function(_, _, snippet)
+        search_context_snippet = snippet
+        pending_fetches = pending_fetches - 1
+        maybe_proceed()
+      end, N)
+    end
+
+    -- If no fetches were initiated, proceed immediately
+    if pending_fetches == 0 then
+      proceed(nil)
+    end
   else
-    check_llm_uncached(task, content)
+    proceed(nil)
   end
 end
 
@@ -1013,19 +1341,98 @@ if opts then
 
   if not settings.prompt then
     if settings.extra_symbols then
-      settings.prompt = "Analyze this email strictly as a spam detector given the email message, subject, " ..
-          "FROM and url domains. Evaluate spam probability (0-1). " ..
+      settings.prompt = "Analyze this email as a spam detector. Evaluate spam probability (0-1).\n\n" ..
+          "LEGITIMATE patterns to recognize:\n" ..
+          "- Verification emails with time-limited codes are NORMAL and legitimate\n" ..
+          "- Transactional emails (receipts, confirmations, password resets) from services\n" ..
+          "- 'Verify email' or 'confirmation code' is NOT automatically phishing\n" ..
+          "- Emails from frequent/known senders (see context) are more trustworthy\n\n" ..
+          "Flag as SPAM/PHISHING only with MULTIPLE red flags:\n" ..
+          "- Urgent threats or fear tactics (account closure, legal action)\n" ..
+          "- Domain impersonation or suspicious lookalikes\n" ..
+          "- Requests for passwords, SSN, credit card numbers\n" ..
+          "- Mismatched URLs pointing to different domains than sender\n" ..
+          "- Poor grammar/spelling in supposedly professional emails\n\n" ..
+          "IMPORTANT: If sender is 'frequent' or 'known', reduce phishing probability " ..
+          "unless there are strong contradictory signals.\n\n" ..
           "Output ONLY 3 lines:\n" ..
           "1. Numeric score (0.00-1.00)\n" ..
-          "2. One-sentence reason citing whether it is spam, the strongest red flag, or why it is ham\n" ..
-          "3. Empty line or mention ONLY the primary concern category if found from the list: " .. 
-            table.concat(lua_util.keys(categories_map), ', ')
+          "2. One-sentence reason citing the strongest indicator\n" ..
+          "3. Primary category if applicable: " ..
+          table.concat(lua_util.keys(categories_map), ', ')
     else
-      settings.prompt = "Analyze this email strictly as a spam detector given the email message, subject, " ..
-          "FROM and url domains. Evaluate spam probability (0-1). " ..
-          "Output ONLY 2 lines:\n" ..
+      settings.prompt = "Analyze this email as a spam detector. Evaluate spam probability (0-1).\n\n" ..
+          "LEGITIMATE patterns to recognize:\n" ..
+          "- Verification emails with time-limited codes are NORMAL and legitimate\n" ..
+          "- Transactional emails (receipts, confirmations, password resets) from services\n" ..
+          "- 'Verify email' or 'confirmation code' is NOT automatically phishing\n" ..
+          "- Emails from frequent/known senders (see context) are more trustworthy\n\n" ..
+          "Flag as SPAM/PHISHING only with MULTIPLE red flags:\n" ..
+          "- Urgent threats or fear tactics (account closure, legal action)\n" ..
+          "- Domain impersonation or suspicious lookalikes\n" ..
+          "- Requests for passwords, SSN, credit card numbers\n" ..
+          "- Mismatched URLs pointing to different domains than sender\n" ..
+          "- Poor grammar/spelling in supposedly professional emails\n\n" ..
+          "IMPORTANT: If sender is 'frequent' or 'known', reduce phishing probability " ..
+          "unless there are strong contradictory signals.\n\n" ..
+          "DO NOT include any additional text or explanation, output 2 lines ONLY:\n" ..
           "1. Numeric score (0.00-1.00)\n" ..
-          "2. One-sentence reason citing whether it is spam, the strongest red flag, or why it is ham\n"
+          "2. One-sentence reason citing the strongest indicator"
+    end
+  end
+
+  -- Compile optional context gating
+  if settings.context then
+    local ctx = settings.context
+    if ctx.enable_expression then
+      local expr = lua_maps_expressions.create(rspamd_config, ctx.enable_expression, N .. "/context-enable")
+      if expr then
+        compiled_context_gating.enable_expr = expr
+      else
+        rspamd_logger.warnx(rspamd_config, 'failed to compile context enable_expression')
+      end
+    end
+    if ctx.disable_expression then
+      local expr = lua_maps_expressions.create(rspamd_config, ctx.disable_expression, N .. "/context-disable")
+      if expr then
+        compiled_context_gating.disable_expr = expr
+      else
+        rspamd_logger.warnx(rspamd_config, 'failed to compile context disable_expression')
+      end
+    end
+    if ctx.enable_map and type(ctx.enable_map) == 'table' and ctx.enable_map.selector and ctx.enable_map.map then
+      local sel = lua_selectors.create_selector_closure(rspamd_config, ctx.enable_map.selector)
+      local map = lua_maps.map_add_from_ucl(ctx.enable_map.map, ctx.enable_map.type or 'set',
+        'GPT context enable map')
+      if sel and map then
+        compiled_context_gating.enable_map = {
+          selector_fn = sel,
+          map = map,
+        }
+      else
+        rspamd_logger.warnx(rspamd_config, 'failed to compile context enable_map: selector or map invalid')
+      end
+    end
+  end
+
+  -- Compile optional search context gating
+  if settings.search_context then
+    local sctx = settings.search_context
+    if sctx.enable_expression then
+      local expr = lua_maps_expressions.create(rspamd_config, sctx.enable_expression, N .. "/search-context-enable")
+      if expr then
+        compiled_search_context_gating.enable_expr = expr
+      else
+        rspamd_logger.warnx(rspamd_config, 'failed to compile search_context enable_expression')
+      end
+    end
+    if sctx.disable_expression then
+      local expr = lua_maps_expressions.create(rspamd_config, sctx.disable_expression, N .. "/search-context-disable")
+      if expr then
+        compiled_search_context_gating.disable_expr = expr
+      else
+        rspamd_logger.warnx(rspamd_config, 'failed to compile search_context disable_expression')
+      end
     end
   end
 end
