@@ -1,26 +1,6 @@
 /*
- The MIT License (MIT)
-
- Copyright (C) 2017 Vsevolod Stakhov <vsevolod@highsecure.ru>
- Copyright (C) 2017 Alexander Moisseev
-
- Permission is hereby granted, free of charge, to any person obtaining a copy
- of this software and associated documentation files (the "Software"), to deal
- in the Software without restriction, including without limitation the rights
- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- copies of the Software, and to permit persons to whom the Software is
- furnished to do so, subject to the following conditions:
-
- The above copyright notice and this permission notice shall be included in
- all copies or substantial portions of the Software.
-
- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- THE SOFTWARE.
+ * Copyright (C) 2017 Vsevolod Stakhov <vsevolod@highsecure.ru>
+ * Copyright (C) 2017 Alexander Moisseev
  */
 
 /* global FooTable */
@@ -112,10 +92,10 @@ define(["jquery", "app/common", "d3evolution", "d3pie", "d3", "footable"],
                     return {
                         label: graph_options.legend.entries[i].label,
                         value: value,
-                        min: Number(yExtents[0].toFixed(6)),
-                        avg: Number(avg.toFixed(6)),
-                        max: Number(yExtents[1].toFixed(6)),
-                        last: Number(curr[curr.length - 1].y.toFixed(6)),
+                        min: Number((yExtents[0] ?? 0).toFixed(6)),
+                        avg: Number((avg ?? 0).toFixed(6)),
+                        max: Number((yExtents[1] ?? 0).toFixed(6)),
+                        last: Number((curr[curr.length - 1]?.y ?? 0).toFixed(6)),
                         color: graph_options.legend.entries[i].color,
                     };
                 }, []);
@@ -128,6 +108,8 @@ define(["jquery", "app/common", "d3evolution", "d3pie", "d3", "footable"],
 
             function initSummaryTable(rows, unit) {
                 common.tables.rrd_summary = FooTable.init("#rrd-table", {
+                    breakpoints: common.breakpoints,
+                    cascade: true,
                     sorting: {
                         enabled: true
                     },
@@ -214,8 +196,13 @@ define(["jquery", "app/common", "d3evolution", "d3pie", "d3", "footable"],
                             if ((curr[0][0].x !== res[0][0].x) ||
                             (curr[0][curr[0].length - 1].x !== res[0][res[0].length - 1].x)) {
                                 time_match = false;
-                                common.alertMessage("alert-error",
-                                    "Neighbours time extents do not match. Check if time is synchronized on all servers.");
+                                common.logError({
+                                    server: "Multi-server",
+                                    endpoint: "graph",
+                                    message: "Neighbours time extents do not match. " +
+                                        "Check if time is synchronized on all servers.",
+                                    errorType: "data_inconsistency"
+                                });
                                 arr.splice(1); // Break out of .reduce() by mutating the source array
                             }
                             return curr;
