@@ -1,25 +1,5 @@
 /*
- The MIT License (MIT)
-
- Copyright (C) 2017 Vsevolod Stakhov <vsevolod@highsecure.ru>
-
- Permission is hereby granted, free of charge, to any person obtaining a copy
- of this software and associated documentation files (the "Software"), to deal
- in the Software without restriction, including without limitation the rights
- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- copies of the Software, and to permit persons to whom the Software is
- furnished to do so, subject to the following conditions:
-
- The above copyright notice and this permission notice shall be included in
- all copies or substantial portions of the Software.
-
- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- THE SOFTWARE.
+ * Copyright (C) 2017 Vsevolod Stakhov <vsevolod@highsecure.ru>
  */
 
 /* global FooTable */
@@ -164,8 +144,12 @@ define(["jquery", "app/common", "app/libft", "footable"],
                     function differentVersions(neighbours_data) {
                         const dv = neighbours_data.some((e) => e.version !== neighbours_data[0].version);
                         if (dv) {
-                            common.alertMessage("alert-danger",
-                                "Neighbours history backend versions do not match. Cannot display history.");
+                            common.logError({
+                                server: "Multi-server",
+                                endpoint: "history",
+                                message: "Neighbours history backend versions do not match. Cannot display history.",
+                                errorType: "data_inconsistency"
+                            });
                             return true;
                         }
                         return false;
@@ -195,15 +179,13 @@ define(["jquery", "app/common", "app/libft", "footable"],
                             version === prevVersion) {
                             common.tables.history.rows.load(items);
                         } else {
-                            libft.destroyTable("history");
-                            // Is there a way to get an event when the table is destroyed?
-                            setTimeout(() => {
+                            libft.destroyTable("history").then(() => {
                                 libft.initHistoryTable(data, items, "history", get_history_columns(data), false,
                                     () => {
                                         $("#history .ft-columns-dropdown .btn-dropdown-apply").removeAttr("disabled");
                                         ui.updateHistoryControlsState();
                                     });
-                            }, 200);
+                            });
                         }
                         prevVersion = version;
                     } else {
@@ -333,7 +315,7 @@ define(["jquery", "app/common", "app/libft", "footable"],
         });
 
         libft.set_page_size("history", $("#history_page_size").val());
-        libft.bindHistoryTableEventHandlers("history", 8);
+        libft.bindHistoryTableEventHandlers("history", 9);
 
         $("#updateHistory").off("click");
         $("#updateHistory").on("click", (e) => {
